@@ -117,7 +117,9 @@ stri_install_icudt <- function(check=TRUE, path=NULL) {
    stopifnot(is.character(path), length(path) == 1)
 
    mirror1 <- "http://static.rexamine.com/packages/"
-   mirror2 <- "http://www.ibspan.waw.pl/~gagolews/stringi/"
+   mirror2 <- "http://www.mini.pw.edu.pl/~gagolews/stringi/"
+   mirror3 <- "http://www.ibspan.waw.pl/~gagolews/stringi/"
+
 
    if (.Platform$endian == 'little') {
       fname <- "icudt52l.zip"
@@ -138,6 +140,7 @@ stri_install_icudt <- function(check=TRUE, path=NULL) {
    message("downloading ICU data library (icudt)")
    allok <- download_from_mirror(mirror1, outfname)
    allok <- allok || download_from_mirror(mirror2, outfname)
+   allok <- allok || download_from_mirror(mirror3, outfname)
 
    if (!allok) {
       message("download failed")
