@@ -31,8 +31,11 @@
 
 
 #include "stri_stringi.h"
-
-
+#include "stri_container_utf8.h"
+#include "stri_container_regex.h"
+#include <deque>
+#include <utility>
+using namespace std;
 
 
 /**
@@ -44,7 +47,7 @@
  * @param first logical - search for the first or the last occurence?
  * @return character vector
  *
- * @version 0.1 (Marek Gagolewski, 2013-06-20)
+ * @version 0.1-?? (Marek Gagolewski, 2013-06-20)
  */
 SEXP stri__extract_firstlast_regex(SEXP str, SEXP pattern, SEXP opts_regex, bool first)
 {
@@ -60,7 +63,7 @@ SEXP stri__extract_firstlast_regex(SEXP str, SEXP pattern, SEXP opts_regex, bool
    StriContainerRegexPattern pattern_cont(pattern, vectorize_length, pattern_flags);
 
    SEXP ret;
-   PROTECT(ret = Rf_allocVector(STRSXP, vectorize_length));
+   STRI__PROTECT(ret = Rf_allocVector(STRSXP, vectorize_length));
 
    for (R_len_t i = pattern_cont.vectorize_init();
          i != pattern_cont.vectorize_end();
@@ -102,7 +105,7 @@ SEXP stri__extract_firstlast_regex(SEXP str, SEXP pattern, SEXP opts_regex, bool
       utext_close(str_text);
       str_text = NULL;
    }
-   UNPROTECT(1);
+   STRI__UNPROTECT_ALL
    return ret;
    STRI__ERROR_HANDLER_END(if (str_text) utext_close(str_text);)
 }
@@ -116,7 +119,7 @@ SEXP stri__extract_firstlast_regex(SEXP str, SEXP pattern, SEXP opts_regex, bool
  * @param opts_regex list
  * @return character vector
  *
- * @version 0.1 (Marek Gagolewski, 2013-06-20)
+ * @version 0.1-?? (Marek Gagolewski, 2013-06-20)
  */
 SEXP stri_extract_first_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 {
@@ -124,7 +127,6 @@ SEXP stri_extract_first_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 }
 
 
-
 /**
  * Extract last occurence of a regex pattern in each string
  *
@@ -133,7 +135,7 @@ SEXP stri_extract_first_regex(SEXP str, SEXP pattern, SEXP opts_regex)
  * @param opts_regex list
  * @return character vector
  *
- * @version 0.1 (Marek Gagolewski, 2013-06-20)
+ * @version 0.1-?? (Marek Gagolewski, 2013-06-20)
  */
 SEXP stri_extract_last_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 {
@@ -141,8 +143,6 @@ SEXP stri_extract_last_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 }
 
 
-
-
 /**
  * Extract all occurences of a regex pattern in each string
  *
@@ -151,7 +151,7 @@ SEXP stri_extract_last_regex(SEXP str, SEXP pattern, SEXP opts_regex)
  * @param opts_regex list
  * @return list of character vectors
  *
- * @version 0.1 (Marek Gagolewski, 2013-06-20)
+ * @version 0.1-?? (Marek Gagolewski, 2013-06-20)
  */
 SEXP stri_extract_all_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 {
@@ -167,7 +167,7 @@ SEXP stri_extract_all_regex(SEXP str, SEXP pattern, SEXP opts_regex)
    StriContainerRegexPattern pattern_cont(pattern, vectorize_length, pattern_flags);
 
    SEXP ret;
-   PROTECT(ret = Rf_allocVector(VECSXP, vectorize_length));
+   STRI__PROTECT(ret = Rf_allocVector(VECSXP, vectorize_length));
 
    for (R_len_t i = pattern_cont.vectorize_init();
          i != pattern_cont.vectorize_end();
@@ -184,9 +184,11 @@ SEXP stri_extract_all_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 
       matcher->reset(str_text);
 
-      deque<R_len_t_x2> occurences;
+      deque< pair<R_len_t, R_len_t> > occurences;
       while ((int)matcher->find()) {
-         occurences.push_back(R_len_t_x2((R_len_t)matcher->start(status), (R_len_t)matcher->end(status)));
+         occurences.push_back(pair<R_len_t, R_len_t>(
+            (R_len_t)matcher->start(status), (R_len_t)matcher->end(status)
+         ));
          if (U_FAILURE(status)) throw StriException(status);
       }
 
@@ -198,21 +200,22 @@ SEXP stri_extract_all_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 
       const char* str_cur_s = str_cont.get(i).c_str();
       SEXP cur_res;
-      PROTECT(cur_res = Rf_allocVector(STRSXP, noccurences));
-      deque<R_len_t_x2>::iterator iter = occurences.begin();
+      STRI__PROTECT(cur_res = Rf_allocVector(STRSXP, noccurences));
+      deque< pair<R_len_t, R_len_t> >::iterator iter = occurences.begin();
       for (R_len_t j = 0; iter != occurences.end(); ++iter, ++j) {
-         R_len_t_x2 curo = *iter;
-         SET_STRING_ELT(cur_res, j, Rf_mkCharLenCE(str_cur_s+curo.v1, curo.v2-curo.v1, CE_UTF8));
+         pair<R_len_t, R_len_t> curo = *iter;
+         SET_STRING_ELT(cur_res, j,
+            Rf_mkCharLenCE(str_cur_s+curo.first, curo.second-curo.first, CE_UTF8));
       }
       SET_VECTOR_ELT(ret, i, cur_res);
-      UNPROTECT(1);
+      STRI__UNPROTECT(1);
    }
 
    if (str_text) {
       utext_close(str_text);
       str_text = NULL;
    }
-   UNPROTECT(1);
+   STRI__UNPROTECT_ALL
    return ret;
    STRI__ERROR_HANDLER_END(if (str_text) utext_close(str_text);)
 }
